<?php
/*
Taken from WordPress Importer plugin
Plugin URI: http://wordpress.org/extend/plugins/wordpress-importer/
Description: Import posts, pages, comments, custom fields, categories, tags and more from a WordPress export file.
Author: wordpressdotorg
Author URI: http://wordpress.org/
License: GPL version 2 or later - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*/
class WXR_Parser {
	function parse( $file ) {
		/* Attempt to use proper XML parsers first*/
		if ( extension_loaded( 'simplexml' ) ) {
			$parser = new WXR_Parser_SimpleXML;
			$result = $parser->parse( $file );

			/* If SimpleXML succeeds or this is an invalid WXR file then return the results*/
			if ( ! is_wp_error( $result ) || 'SimpleXML_parse_error' != $result->get_error_code() ) {
				return $result;
			}
		} elseif ( extension_loaded( 'xml' ) ) {
			$parser = new WXR_Parser_XML;
			$result = $parser->parse( $file );

			/* If XMLParser succeeds or this is an invalid WXR file then return the results*/
			if ( ! is_wp_error( $result ) || 'XML_parse_error' != $result->get_error_code() ) {
				return $result;
			}
		}

		/* We have a malformed XML file, so display the error and fallthrough to regex*/
		if ( isset( $result ) && defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
			echo '<pre>';
			if ( 'SimpleXML_parse_error' == $result->get_error_code() ) {
				foreach ( $result->get_error_data() as $error ) {
					echo $error->line . ':' . $error->column . ' ' . esc_html( $error->message ) . "\n";
				}
			} elseif ( 'XML_parse_error' == $result->get_error_code() ) {
				$error = $result->get_error_data();
				echo $error[0] . ':' . $error[1] . ' ' . esc_html( $error[2] );
			}
			echo '</pre>';
			echo '<p><strong>' . __( 'There was an error when reading this WXR file', 'templatic-admin' ) . '</strong><br />';
			echo __( 'Details are shown above. The importer will now try again with a different parser...', 'templatic-admin' ) . '</p>';
		}

		/* use regular expressions if nothing else available or this is bad XML*/
		$parser = new WXR_Listing_Parser_Regex;
		return $parser->parse( $file );
	}
}

/**
 * WXR Parser that makes use of the SimpleXML PHP extension.
 */
class WXR_Parser_SimpleXML {
	function parse( $file ) {
		$authors = $posts = $categories = $tags = $terms = array();

		$internal_errors = libxml_use_internal_errors( true );

		$dom = new DOMDocument;
		$old_value = null;
		if ( function_exists( 'libxml_disable_entity_loader' ) ) {
			$old_value = libxml_disable_entity_loader( true );
		}
		$success = $dom->loadXML( file_get_contents( $file ) );
		if ( ! is_null( $old_value ) ) {
			libxml_disable_entity_loader( $old_value );
		}

		if ( ! $success || isset( $dom->doctype ) ) {
			return new WP_Error( 'SimpleXML_parse_error', __( 'There was an error when reading this WXR file', 'templatic-admin' ), libxml_get_errors() );
		}

		$xml = simplexml_import_dom( $dom );
		unset( $dom );

		/* halt if loading produces an error*/
		if ( ! $xml ) {
			return new WP_Error( 'SimpleXML_parse_error', __( 'There was an error when reading this WXR file', 'templatic-admin' ), libxml_get_errors() );
		}

		$wxr_version = $xml->xpath( '/rss/channel/wp:wxr_version' );
		if ( ! $wxr_version ) {
			return new WP_Error( 'WXR_parse_error', __( 'This does not appear to be a WXR file, missing/invalid WXR version number', 'templatic-admin' ) );
		}

		$wxr_version = (string) trim( $wxr_version[0] );
		/* confirm that we are dealing with the correct file format*/
		if ( ! preg_match( '/^\d+\.\d+$/', $wxr_version ) ) {
			return new WP_Error( 'WXR_parse_error', __( 'This does not appear to be a WXR file, missing/invalid WXR version number', 'templatic-admin' ) );
		}

		$base_url = $xml->xpath( '/rss/channel/wp:base_site_url' );
		$base_url = (string) trim( $base_url[0] );

		$namespaces = $xml->getDocNamespaces();
		if ( ! isset( $namespaces['wp'] ) ) {
			$namespaces['wp'] = 'http://wordpress.org/export/1.1/';
		}
		if ( ! isset( $namespaces['excerpt'] ) ) {
			$namespaces['excerpt'] = 'http://wordpress.org/export/1.1/excerpt/';
		}

		/* grab authors*/
		foreach ( $xml->xpath( '/rss/channel/wp:author' ) as $author_arr ) {
			$a = $author_arr->children( $namespaces['wp'] );
			$login = (string) $a->author_login;
			$authors[ $login ] = array(
				'author_id' => (int) $a->author_id,
				'author_login' => $login,
				'author_email' => (string) $a->author_email,
				'author_display_name' => (string) $a->author_display_name,
				'author_first_name' => (string) $a->author_first_name,
				'author_last_name' => (string) $a->author_last_name,
				);
		}

		/* grab cats, tags and terms*/
		foreach ( $xml->xpath( '/rss/channel/wp:category' ) as $term_arr ) {
			$t = $term_arr->children( $namespaces['wp'] );
			$categories[] = array(
				'term_id' => (int) $t->term_id,
				'category_nicename' => (string) $t->category_nicename,
				'category_parent' => (string) $t->category_parent,
				'cat_name' => (string) $t->cat_name,
				'category_description' => (string) $t->category_description,
				);
		}

		foreach ( $xml->xpath( '/rss/channel/wp:tag' ) as $term_arr ) {
			$t = $term_arr->children( $namespaces['wp'] );
			$tags[] = array(
				'term_id' => (int) $t->term_id,
				'tag_slug' => (string) $t->tag_slug,
				'tag_name' => (string) $t->tag_name,
				'tag_description' => (string) $t->tag_description,
				);
		}

		foreach ( $xml->xpath( '/rss/channel/wp:term' ) as $term_arr ) {
			$t = $term_arr->children( $namespaces['wp'] );
			$terms[] = array(
				'term_id' => (int) $t->term_id,
				'term_taxonomy' => (string) $t->term_taxonomy,
				'slug' => (string) $t->term_slug,
				'term_parent' => (string) $t->term_parent,
				'term_name' => (string) $t->term_name,
				'term_description' => (string) $t->term_description,
				);
		}

		/* grab posts*/
		foreach ( $xml->channel->item as $item ) {
			$post = array(
				'post_title' => (string) $item->title,
				'guid' => (string) $item->guid,
				);

			$dc = $item->children( 'http://purl.org/dc/elements/1.1/' );
			$post['post_author'] = (string) $dc->creator;

			$content = $item->children( 'http://purl.org/rss/1.0/modules/content/' );
			$excerpt = $item->children( $namespaces['excerpt'] );
			$post['post_content'] = (string) $content->encoded;
			$post['post_excerpt'] = (string) $excerpt->encoded;

			$wp = $item->children( $namespaces['wp'] );
			$post['post_id'] = (int) $wp->post_id;
			$post['post_date'] = (string) $wp->post_date;
			$post['post_date_gmt'] = (string) $wp->post_date_gmt;
			$post['comment_status'] = (string) $wp->comment_status;
			$post['ping_status'] = (string) $wp->ping_status;
			$post['post_name'] = (string) $wp->post_name;
			$post['status'] = (string) $wp->status;
			$post['post_parent'] = (int) $wp->post_parent;
			$post['menu_order'] = (int) $wp->menu_order;
			$post['post_type'] = (string) $wp->post_type;
			$post['post_password'] = (string) $wp->post_password;
			$post['is_sticky'] = (int) $wp->is_sticky;

			if ( isset( $wp->attachment_url ) ) {
				$post['attachment_url'] = (string) $wp->attachment_url;
			}

			foreach ( $item->category as $c ) {
				$att = $c->attributes();
				if ( isset( $att['nicename'] ) ) {
					$post['terms'][] = array(
						'name' => (string) $c,
						'slug' => (string) $att['nicename'],
						'domain' => (string) $att['domain'],
						);
				}
			}

			foreach ( $wp->postmeta as $meta ) {
				$post['postmeta'][] = array(
					'key' => (string) $meta->meta_key,
					'value' => (string) $meta->meta_value,
					);
			}

			foreach ( $wp->comment as $comment ) {
				$meta = array();
				if ( isset( $comment->commentmeta ) ) {
					foreach ( $comment->commentmeta as $m ) {
						$meta[] = array(
							'key' => (string) $m->meta_key,
							'value' => (string) $m->meta_value,
							);
					}
				}

				$post['comments'][] = array(
					'comment_id' => (int) $comment->comment_id,
					'comment_author' => (string) $comment->comment_author,
					'comment_author_email' => (string) $comment->comment_author_email,
					'comment_author_IP' => (string) $comment->comment_author_IP,
					'comment_author_url' => (string) $comment->comment_author_url,
					'comment_date' => (string) $comment->comment_date,
					'comment_date_gmt' => (string) $comment->comment_date_gmt,
					'comment_content' => (string) $comment->comment_content,
					'comment_approved' => (string) $comment->comment_approved,
					'comment_type' => (string) $comment->comment_type,
					'comment_parent' => (string) $comment->comment_parent,
					'comment_user_id' => (int) $comment->comment_user_id,
					'commentmeta' => $meta,
					);
			}

			$posts[] = $post;
		}// End foreach().

		return array(
			'authors' => $authors,
			'posts' => $posts,
			'categories' => $categories,
			'tags' => $tags,
			'terms' => $terms,
			'base_url' => $base_url,
			'version' => $wxr_version,
			);
	}
}

/**
* WXR Parser that makes use of the XML Parser PHP extension.
*/
class WXR_Parser_XML {
	var $wp_tags = array(
		'wp:post_id',
	'wp:post_date',
	'wp:post_date_gmt',
	'wp:comment_status',
	'wp:ping_status',
	'wp:attachment_url',
		'wp:status',
	'wp:post_name',
	'wp:post_parent',
	'wp:menu_order',
	'wp:post_type',
	'wp:post_password',
		'wp:is_sticky',
	'wp:term_id',
	'wp:category_nicename',
	'wp:category_parent',
	'wp:cat_name',
	'wp:category_description',
		'wp:tag_slug',
	'wp:tag_name',
	'wp:tag_description',
	'wp:term_taxonomy',
	'wp:term_parent',
		'wp:term_name',
	'wp:term_description',
	'wp:author_id',
	'wp:author_login',
	'wp:author_email',
	'wp:author_display_name',
		'wp:author_first_name',
	'wp:author_last_name',
		);
	var $wp_sub_tags = array(
		'wp:comment_id',
	'wp:comment_author',
	'wp:comment_author_email',
	'wp:comment_author_url',
		'wp:comment_author_IP',
	'wp:comment_date',
	'wp:comment_date_gmt',
	'wp:comment_content',
		'wp:comment_approved',
	'wp:comment_type',
	'wp:comment_parent',
	'wp:comment_user_id',
		);

	function parse( $file ) {
		$this->wxr_version = $this->in_post = $this->cdata = $this->data = $this->sub_data = $this->in_tag = $this->in_sub_tag = false;
		$this->authors = $this->posts = $this->term = $this->category = $this->tag = array();

		$xml = xml_parser_create( 'UTF-8' );
		xml_parser_set_option( $xml, XML_OPTION_SKIP_WHITE, 1 );
		xml_parser_set_option( $xml, XML_OPTION_CASE_FOLDING, 0 );
		xml_set_object( $xml, $this );
		xml_set_character_data_handler( $xml, 'cdata' );
		xml_set_element_handler( $xml, 'tag_open', 'tag_close' );

		if ( ! xml_parse( $xml, file_get_contents( $file ), true ) ) {
			$current_line = xml_get_current_line_number( $xml );
			$current_column = xml_get_current_column_number( $xml );
			$error_code = xml_get_error_code( $xml );
			$error_string = xml_error_string( $error_code );
			return new WP_Error( 'XML_parse_error', 'There was an error when reading this WXR file', array( $current_line, $current_column, $error_string ) );
		}
		xml_parser_free( $xml );

		if ( ! preg_match( '/^\d+\.\d+$/', $this->wxr_version ) ) {
			return new WP_Error( 'WXR_parse_error', __( 'This does not appear to be a WXR file, missing/invalid WXR version number', 'templatic-admin' ) );
		}

		return array(
			'authors' => $this->authors,
			'posts' => $this->posts,
			'categories' => $this->category,
			'tags' => $this->tag,
			'terms' => $this->term,
			'base_url' => $this->base_url,
			'version' => $this->wxr_version,
			);
	}

	function tag_open( $parse, $tag, $attr ) {
		if ( in_array( $tag, $this->wp_tags ) ) {
			$this->in_tag = substr( $tag, 3 );
			return;
		}

		if ( in_array( $tag, $this->wp_sub_tags ) ) {
			$this->in_sub_tag = substr( $tag, 3 );
			return;
		}

		switch ( $tag ) {
			case 'category':
				if ( isset( $attr['domain'], $attr['nicename'] ) ) {
					$this->sub_data['domain'] = $attr['domain'];
					$this->sub_data['slug'] = $attr['nicename'];
				}
			break;
			case 'item': $this->in_post = true;
			case 'title': if ( $this->in_post ) { $this->in_tag = 'post_title';
			} break;
			case 'guid': $this->in_tag = 'guid';
break;
			case 'dc:creator': $this->in_tag = 'post_author';
break;
			case 'content:encoded': $this->in_tag = 'post_content';
break;
			case 'excerpt:encoded': $this->in_tag = 'post_excerpt';
break;

			case 'wp:term_slug': $this->in_tag = 'slug';
break;
			case 'wp:meta_key': $this->in_sub_tag = 'key';
break;
			case 'wp:meta_value': $this->in_sub_tag = 'value';
break;
		}
	}

	function cdata( $parser, $cdata ) {
		if ( ! trim( $cdata ) ) {
			return;
		}

		$this->cdata .= trim( $cdata );
	}

	function tag_close( $parser, $tag ) {
		switch ( $tag ) {
			case 'wp:comment':
				unset( $this->sub_data['key'], $this->sub_data['value'] ); /* remove meta sub_data*/
				if ( ! empty( $this->sub_data ) ) {
					$this->data['comments'][] = $this->sub_data;
				}
				$this->sub_data = false;
			break;
			case 'wp:commentmeta':
				$this->sub_data['commentmeta'][] = array(
				'key' => $this->sub_data['key'],
				'value' => $this->sub_data['value'],
				);
			break;
			case 'category':
				if ( ! empty( $this->sub_data ) ) {
					$this->sub_data['name'] = $this->cdata;
					$this->data['terms'][] = $this->sub_data;
				}
				$this->sub_data = false;
			break;
			case 'wp:postmeta':
				if ( ! empty( $this->sub_data ) ) {
					$this->data['postmeta'][] = $this->sub_data;
				}
				$this->sub_data = false;
			break;
			case 'item':
				$this->posts[] = $this->data;
				$this->data = false;
			break;
			case 'wp:category':
			case 'wp:tag':
			case 'wp:term':
				$n = substr( $tag, 3 );
				array_push( $this->$n, $this->data );
				$this->data = false;
			break;
			case 'wp:author':
				if ( ! empty( $this->data['author_login'] ) ) {
					$this->authors[ $this->data['author_login'] ] = $this->data;
				}
				$this->data = false;
			break;
			case 'wp:base_site_url':
				$this->base_url = $this->cdata;
			break;
			case 'wp:wxr_version':
				$this->wxr_version = $this->cdata;
			break;

			default:
				if ( $this->in_sub_tag ) {
					$this->sub_data[ $this->in_sub_tag ] = ! empty( $this->cdata ) ? $this->cdata : '';
					$this->in_sub_tag = false;
				} elseif ( $this->in_tag ) {
					$this->data[ $this->in_tag ] = ! empty( $this->cdata ) ? $this->cdata : '';
					$this->in_tag = false;
				}
		}// End switch().

		$this->cdata = false;
	}
}

/**
* WXR Parser that uses regular expressions. Fallback for installs without an XML parser.
*/
class WXR_Listing_Parser_Regex {
	var $authors = array();
	var $posts = array();
	var $categories = array();
	var $tags = array();
	var $terms = array();
	var $base_url = '';

	function WXR_Listing_Parser_Regex() {
		$this->__construct();
	}

	function __construct() {
		$this->has_gzip = is_callable( 'gzopen' );
	}

	function parse( $file ) {
		$wxr_version = $in_post = false;

		$fp = $this->fopen( $file, 'r' );
		if ( $fp ) {
			while ( ! $this->feof( $fp ) ) {
				$importline = rtrim( $this->fgets( $fp ) );

				if ( ! $wxr_version && preg_match( '|<wp:wxr_version>(\d+\.\d+)</wp:wxr_version>|', $importline, $version ) ) {
					$wxr_version = $version[1];
				}

				if ( false !== strpos( $importline, '<wp:base_site_url>' ) ) {
					preg_match( '|<wp:base_site_url>(.*?)</wp:base_site_url>|is', $importline, $url );
					$this->base_url = $url[1];
					continue;
				}
				if ( false !== strpos( $importline, '<wp:category>' ) ) {
					preg_match( '|<wp:category>(.*?)</wp:category>|is', $importline, $category );
					$this->categories[] = $this->process_category( $category[1] );
					continue;
				}
				if ( false !== strpos( $importline, '<wp:tag>' ) ) {
					preg_match( '|<wp:tag>(.*?)</wp:tag>|is', $importline, $tag );
					$this->tags[] = $this->process_tag( $tag[1] );
					continue;
				}
				if ( false !== strpos( $importline, '<wp:term>' ) ) {
					preg_match( '|<wp:term>(.*?)</wp:term>|is', $importline, $term );
					$this->terms[] = $this->process_term( $term[1] );
					continue;
				}
				if ( false !== strpos( $importline, '<wp:author>' ) ) {
					preg_match( '|<wp:author>(.*?)</wp:author>|is', $importline, $author );
					$a = $this->process_author( $author[1] );
					$this->authors[ $a['author_login'] ] = $a;
					continue;
				}
				if ( false !== strpos( $importline, '<item>' ) ) {
					$post = '';
					$in_post = true;
					continue;
				}
				if ( false !== strpos( $importline, '</item>' ) ) {
					$in_post = false;
					$this->posts[] = $this->process_post( $post );
					continue;
				}
				if ( $in_post ) {
					$post .= $importline . "\n";
				}
			}// End while().

			$this->fclose( $fp );
		}// End if().

		if ( ! $wxr_version ) {
			return new WP_Error( 'WXR_parse_error', __( 'This does not appear to be a WXR file, missing/invalid WXR version number', 'templatic-admin' ) );
		}

		return array(
			'authors' => $this->authors,
			'posts' => $this->posts,
			'categories' => $this->categories,
			'tags' => $this->tags,
			'terms' => $this->terms,
			'base_url' => $this->base_url,
			'version' => $wxr_version,
			);
	}

	function get_tag( $string, $tag ) {
		preg_match( "|<$tag.*?>(.*?)</$tag>|is", $string, $return );
		if ( isset( $return[1] ) ) {
			if ( substr( $return[1], 0, 9 ) == '<![CDATA[' ) {
				if ( strpos( $return[1], ']]]]><![CDATA[>' ) !== false ) {
					preg_match_all( '|<!\[CDATA\[(.*?)\]\]>|s', $return[1], $matches );
					$return = '';
					foreach ( $matches[1] as $match ) {
						$return .= $match;
					}
				} else {
					$return = preg_replace( '|^<!\[CDATA\[(.*)\]\]>$|s', '$1', $return[1] );
				}
			} else {
				$return = $return[1];
			}
		} else {
			$return = '';
		}
		return $return;
	}

	function process_category( $c ) {
		return array(
			'term_id' => $this->get_tag( $c, 'wp:term_id' ),
			'cat_name' => $this->get_tag( $c, 'wp:cat_name' ),
			'category_nicename'	=> $this->get_tag( $c, 'wp:category_nicename' ),
			'category_parent' => $this->get_tag( $c, 'wp:category_parent' ),
			'category_description' => $this->get_tag( $c, 'wp:category_description' ),
			);
	}

	function process_tag( $t ) {
		return array(
			'term_id' => $this->get_tag( $t, 'wp:term_id' ),
			'tag_name' => $this->get_tag( $t, 'wp:tag_name' ),
			'tag_slug' => $this->get_tag( $t, 'wp:tag_slug' ),
			'tag_description' => $this->get_tag( $t, 'wp:tag_description' ),
			);
	}

	function process_term( $t ) {
		return array(
			'term_id' => $this->get_tag( $t, 'wp:term_id' ),
			'term_taxonomy' => $this->get_tag( $t, 'wp:term_taxonomy' ),
			'slug' => $this->get_tag( $t, 'wp:term_slug' ),
			'term_parent' => $this->get_tag( $t, 'wp:term_parent' ),
			'term_name' => $this->get_tag( $t, 'wp:term_name' ),
			'term_description' => $this->get_tag( $t, 'wp:term_description' ),
			);
	}

	function process_author( $a ) {
		return array(
			'author_id' => $this->get_tag( $a, 'wp:author_id' ),
			'author_login' => $this->get_tag( $a, 'wp:author_login' ),
			'author_email' => $this->get_tag( $a, 'wp:author_email' ),
			'author_display_name' => $this->get_tag( $a, 'wp:author_display_name' ),
			'author_first_name' => $this->get_tag( $a, 'wp:author_first_name' ),
			'author_last_name' => $this->get_tag( $a, 'wp:author_last_name' ),
			);
	}

	function process_post( $post ) {
		$post_id        = $this->get_tag( $post, 'wp:post_id' );
		$post_title     = $this->get_tag( $post, 'title' );
		$post_date      = $this->get_tag( $post, 'wp:post_date' );
		$post_date_gmt  = $this->get_tag( $post, 'wp:post_date_gmt' );
		$comment_status = $this->get_tag( $post, 'wp:comment_status' );
		$ping_status    = $this->get_tag( $post, 'wp:ping_status' );
		$status         = $this->get_tag( $post, 'wp:status' );
		$post_name      = $this->get_tag( $post, 'wp:post_name' );
		$post_parent    = $this->get_tag( $post, 'wp:post_parent' );
		$menu_order     = $this->get_tag( $post, 'wp:menu_order' );
		$post_type      = $this->get_tag( $post, 'wp:post_type' );
		$post_password  = $this->get_tag( $post, 'wp:post_password' );
		$is_sticky      = $this->get_tag( $post, 'wp:is_sticky' );
		$guid           = $this->get_tag( $post, 'guid' );
		$post_author    = $this->get_tag( $post, 'dc:creator' );

		$post_excerpt = $this->get_tag( $post, 'excerpt:encoded' );
		$post_excerpt = preg_replace_callback( '|<(/?[A-Z]+)|', array( &$this, '_normalize_tag' ), $post_excerpt );
		$post_excerpt = str_replace( '<br>', '<br />', $post_excerpt );
		$post_excerpt = str_replace( '<hr>', '<hr />', $post_excerpt );

		$post_content = $this->get_tag( $post, 'content:encoded' );
		$post_content = preg_replace_callback( '|<(/?[A-Z]+)|', array( &$this, '_normalize_tag' ), $post_content );
		$post_content = str_replace( '<br>', '<br />', $post_content );
		$post_content = str_replace( '<hr>', '<hr />', $post_content );

		$postdata = compact( 'post_id', 'post_author', 'post_date', 'post_date_gmt', 'post_content', 'post_excerpt',
			'post_title', 'status', 'post_name', 'comment_status', 'ping_status', 'guid', 'post_parent',
			'menu_order', 'post_type', 'post_password', 'is_sticky'
		);

		$attachment_url = $this->get_tag( $post, 'wp:attachment_url' );
		if ( $attachment_url ) {
			$postdata['attachment_url'] = $attachment_url;
		}

		preg_match_all( '|<category domain="([^"]+?)" nicename="([^"]+?)">(.+?)</category>|is', $post, $terms, PREG_SET_ORDER );
		foreach ( $terms as $t ) {
			$post_terms[] = array(
				'slug' => $t[2],
				'domain' => $t[1],
				'name' => str_replace( array( '<![CDATA[', ']]>' ), '', $t[3] ),
				);
		}
		if ( ! empty( $post_terms ) ) { $postdata['terms'] = $post_terms;
		}

		preg_match_all( '|<wp:comment>(.+?)</wp:comment>|is', $post, $comments );
		$comments = $comments[1];
		if ( $comments ) {
			foreach ( $comments as $comment ) {
				preg_match_all( '|<wp:commentmeta>(.+?)</wp:commentmeta>|is', $comment, $commentmeta );
				$commentmeta = $commentmeta[1];
				$c_meta = array();
				foreach ( $commentmeta as $m ) {
					$c_meta[] = array(
						'key' => $this->get_tag( $m, 'wp:meta_key' ),
						'value' => $this->get_tag( $m, 'wp:meta_value' ),
						);
				}

				$post_comments[] = array(
					'comment_id' => $this->get_tag( $comment, 'wp:comment_id' ),
					'comment_author' => $this->get_tag( $comment, 'wp:comment_author' ),
					'comment_author_email' => $this->get_tag( $comment, 'wp:comment_author_email' ),
					'comment_author_IP' => $this->get_tag( $comment, 'wp:comment_author_IP' ),
					'comment_author_url' => $this->get_tag( $comment, 'wp:comment_author_url' ),
					'comment_date' => $this->get_tag( $comment, 'wp:comment_date' ),
					'comment_date_gmt' => $this->get_tag( $comment, 'wp:comment_date_gmt' ),
					'comment_content' => $this->get_tag( $comment, 'wp:comment_content' ),
					'comment_approved' => $this->get_tag( $comment, 'wp:comment_approved' ),
					'comment_type' => $this->get_tag( $comment, 'wp:comment_type' ),
					'comment_parent' => $this->get_tag( $comment, 'wp:comment_parent' ),
					'comment_user_id' => $this->get_tag( $comment, 'wp:comment_user_id' ),
					'commentmeta' => $c_meta,
					);
			}
		}
		if ( ! empty( $post_comments ) ) { $postdata['comments'] = $post_comments;
		}

		preg_match_all( '|<wp:postmeta>(.+?)</wp:postmeta>|is', $post, $postmeta );
		$postmeta = $postmeta[1];
		if ( $postmeta ) {
			foreach ( $postmeta as $p ) {
				$post_postmeta[] = array(
					'key' => $this->get_tag( $p, 'wp:meta_key' ),
					'value' => $this->get_tag( $p, 'wp:meta_value' ),
					);
			}
		}
		if ( ! empty( $post_postmeta ) ) { $postdata['postmeta'] = $post_postmeta;
		}

		return $postdata;
	}

	function _normalize_tag( $matches ) {
		return '<' . strtolower( $matches[1] );
	}

	function fopen( $filename, $mode = 'r' ) {
		if ( $this->has_gzip ) {
			return gzopen( $filename, $mode );
		}
		return fopen( $filename, $mode );
	}

	function feof( $fp ) {
		if ( $this->has_gzip ) {
			return gzeof( $fp );
		}
		return feof( $fp );
	}

	function fgets( $fp, $len = 8192 ) {
		if ( $this->has_gzip ) {
			return gzgets( $fp, $len );
		}
		return fgets( $fp, $len );
	}

	function fclose( $fp ) {
		if ( $this->has_gzip ) {
			return gzclose( $fp );
		}
		return fclose( $fp );
	}
}

define( 'IMPORT_DEBUG', false );

/* Load Importer API*/
require_once ABSPATH . 'wp-admin/includes/import.php';

if ( ! class_exists( 'WP_Importer' ) ) {
	$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
	if ( file_exists( $class_wp_importer ) ) {
		require $class_wp_importer;
	}
}

/**
* WordPress Importer class for managing the import process of a WXR file
*
* @package WordPress
* @subpackage Importer
*/
if ( class_exists( 'WP_Importer' ) ) {
	class Listing_WP_Import extends WP_Importer {
		var $max_wxr_version = 1.2; /* max. supported WXR version*/
		var $id; /* WXR attachment ID*/
		/* information to import from WXR file*/
		var $version;
		var $authors = array();
		var $posts = array();
		var $terms = array();
		var $categories = array();
		var $tags = array();
		var $base_url = '';
		/* mappings from old information to new*/
		var $processed_authors = array();
		var $author_mapping = array();
		var $processed_terms = array();
		var $processed_posts = array();
		var $post_orphans = array();
		var $processed_menu_items = array();
		var $menu_item_orphans = array();
		var $missing_menu_items = array();
		var $fetch_attachments = true;
		var $url_remap = array();
		var $featured_images = array();
		function Listing_WP_Import() {
			/* nothing */ }
		function import( $file ) {
			add_filter( 'import_post_meta_key', array( $this, 'is_valid_meta_key' ) );
			add_filter( 'http_request_timeout', array( &$this, 'bump_request_timeout' ) );
			$this->import_start( $file );
			$this->get_author_mapping();
			wp_suspend_cache_invalidation( true );
			$this->process_categories();
			$this->process_tags();
			$this->process_terms();
			$this->process_posts();
			wp_suspend_cache_invalidation( false );
			/* update incorrect/missing information in the DB*/
			$this->backfill_parents();
			$this->backfill_attachment_urls();
			$this->remap_featured_images();
			$this->import_end();
		}
		function import_start( $file ) {
			$import_data = $this->parse( $file );
			$this->version = $import_data['version'];
			$this->get_authors_from_import( $import_data );
			$this->posts = $import_data['posts'];
			$this->terms = $import_data['terms'];
			$this->categories = $import_data['categories'];
			$this->tags = $import_data['tags'];
			$this->base_url = esc_url( $import_data['base_url'] );
			wp_defer_term_counting( true );
			wp_defer_comment_counting( true );
			do_action( 'import_start' );
		}
		function import_end() {
			wp_import_cleanup( $this->id );
			wp_cache_flush();
			foreach ( get_taxonomies() as $tax ) {
				delete_option( "{$tax}_children" );
				_get_term_hierarchy( $tax );
			}
			wp_defer_term_counting( false );
			wp_defer_comment_counting( false );
			do_action( 'import_end' );
		}
		function handle_upload() {
			$file = wp_import_handle_upload();
			$this->id = (int) $file['id'];
			$import_data = $this->parse( $file['file'] );
			$this->version = $import_data['version'];
			$this->get_authors_from_import( $import_data );
			return true;
		}
		function get_authors_from_import( $import_data ) {
			if ( ! empty( $import_data['authors'] ) ) {
				$this->authors = $import_data['authors'];
				/* no author information, grab it from the posts*/
			} else {
				foreach ( $import_data['posts'] as $post ) {
					$login = sanitize_user( $post['post_author'], true );
					if ( empty( $login ) ) {
						printf( __( 'Failed to import author %s. Their posts will be attributed to the current user.', 'templatic-admin' ), esc_html( $post['post_author'] ) );
						echo '<br />';
						continue;
					}

					if ( ! isset( $this->authors[ $login ] ) ) {
						$this->authors[ $login ] = array(
						'author_login' => $login,
						'author_display_name' => $post['post_author'],
						);
					}
				}
			}
		}
		function author_select( $n, $author ) {
			echo __( 'Import author:', 'templatic-admin' );
			echo ' <strong>' . esc_html( $author['author_display_name'] );
			if ( $this->version != '1.0' ) { echo ' (' . esc_html( $author['author_login'] ) . ')';
			}
			echo '</strong><br />';
			if ( $this->version != '1.0' ) {
				echo '<div style="margin-left:18px">';
			}
			$create_users = $this->allow_create_users();
			if ( $create_users ) {
				if ( $this->version != '1.0' ) {
					echo __( 'or create new user with login name:', 'templatic-admin' );
					$value = '';
				} else {
					echo __( 'as a new user:', 'templatic-admin' );
					$value = esc_attr( sanitize_user( $author['author_login'], true ) );
				}

				echo ' <input type="text" name="user_new[' . $n . ']" value="' . $value . '" /><br />';
			}
			if ( ! $create_users && $this->version == '1.0' ) {
				echo __( 'assign posts to an existing user:', 'templatic-admin' );
			} else { 			echo __( 'or assign posts to an existing user:', 'templatic-admin' );
			}
			wp_dropdown_users( array(
				'name' => "user_map[$n]",
				'multi' => true,
				'show_option_all' => __( '- Select -', 'templatic-admin' ),
			) );
			echo '<input type="hidden" name="imported_authors[' . $n . ']" value="' . esc_attr( $author['author_login'] ) . '" />';

			if ( $this->version != '1.0' ) {
				echo '</div>';
			}
		}
		function get_author_mapping() {
			if ( ! isset( $_POST['imported_authors'] ) ) {
				return;
			}
			$create_users = $this->allow_create_users();
			foreach ( (array) $_POST['imported_authors'] as $i => $old_login ) {
				/* Multisite adds strtolower to sanitize_user. Need to sanitize here to stop breakage in process_posts.*/
				$santized_old_login = sanitize_user( $old_login, true );
				$old_id = isset( $this->authors[ $old_login ]['author_id'] ) ? intval( $this->authors[ $old_login ]['author_id'] ) : false;
				if ( ! empty( $_POST['user_map'][ $i ] ) ) {
					$user = get_userdata( intval( $_POST['user_map'][ $i ] ) );
					if ( isset( $user->ID ) ) {
						if ( $old_id ) {
							$this->processed_authors[ $old_id ] = $user->ID;
						}
						$this->author_mapping[ $santized_old_login ] = $user->ID;
					}
				} elseif ( $create_users ) {
					if ( ! empty( $_POST['user_new'][ $i ] ) ) {
						$user_id = wp_create_user( $_POST['user_new'][ $i ], wp_generate_password() );
					} elseif ( $this->version != '1.0' ) {
						$user_data = array(
						'user_login' => $old_login,
						'user_pass' => wp_generate_password(),
						'user_email' => isset( $this->authors[ $old_login ]['author_email'] ) ? $this->authors[ $old_login ]['author_email'] : '',
						'display_name' => $this->authors[ $old_login ]['author_display_name'],
						'first_name' => isset( $this->authors[ $old_login ]['author_first_name'] ) ? $this->authors[ $old_login ]['author_first_name'] : '',
						'last_name' => isset( $this->authors[ $old_login ]['author_last_name'] ) ? $this->authors[ $old_login ]['author_last_name'] : '',
						);
						$user_id = wp_insert_user( $user_data );
					}

					if ( ! is_wp_error( $user_id ) ) {
						if ( $old_id ) {
							$this->processed_authors[ $old_id ] = $user_id;
						}
						$this->author_mapping[ $santized_old_login ] = $user_id;
					} else {
						printf( __( 'Failed to create new user for %s. Their posts will be attributed to the current user.', 'templatic-admin' ), esc_html( $this->authors[ $old_login ]['author_display_name'] ) );
						if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
							echo ' ' . $user_id->get_error_message();
						}
						echo '<br />';
					}
				}// End if().

				/* failsafe: if the user_id was invalid, default to the current user*/
				if ( ! isset( $this->author_mapping[ $santized_old_login ] ) ) {
					if ( $old_id ) {
						$this->processed_authors[ $old_id ] = (int) get_current_user_id();
					}
					$this->author_mapping[ $santized_old_login ] = (int) get_current_user_id();
				}
			}// End foreach().
		}
		function process_categories() {
			$this->categories = apply_filters( 'wp_import_categories', $this->categories );
			if ( empty( $this->categories ) ) {
				return;
			}
			foreach ( $this->categories as $cat ) {
				/* if the category already exists leave it alone*/
				$term_id = term_exists( $cat['category_nicename'], 'category' );
				if ( $term_id ) {
					if ( is_array( $term_id ) ) { $term_id = $term_id['term_id'];
					}
					if ( isset( $cat['term_id'] ) ) {
						$this->processed_terms[ intval( $cat['term_id'] ) ] = (int) $term_id;
					}
					continue;
				}
				$category_parent = empty( $cat['category_parent'] ) ? 0 : category_exists( $cat['category_parent'] );
				$category_description = isset( $cat['category_description'] ) ? $cat['category_description'] : '';
				$catarr = array(
				'category_nicename' => $cat['category_nicename'],
				'category_parent' => $category_parent,
				'cat_name' => $cat['cat_name'],
				'category_description' => $category_description,
					);

				$id = wp_insert_category( $catarr );
				if ( ! is_wp_error( $id ) ) {
					if ( isset( $cat['term_id'] ) ) {
						$this->processed_terms[ intval( $cat['term_id'] ) ] = $id;
					}
				}
			}
			unset( $this->categories );
		}
		function process_tags() {
			$this->tags = apply_filters( 'wp_import_tags', $this->tags );
			if ( empty( $this->tags ) ) {
				return;
			}
			foreach ( $this->tags as $tag ) {
				/* if the tag already exists leave it alone*/
				$term_id = term_exists( $tag['tag_slug'], 'post_tag' );
				if ( $term_id ) {
					if ( is_array( $term_id ) ) { $term_id = $term_id['term_id'];
					}
					if ( isset( $tag['term_id'] ) ) {
						$this->processed_terms[ intval( $tag['term_id'] ) ] = (int) $term_id;
					}
					continue;
				}
				$tag_desc = isset( $tag['tag_description'] ) ? $tag['tag_description'] : '';
				$tagarr = array(
					'slug' => $tag['tag_slug'],
					'description' => $tag_desc,
				);
				$id = wp_insert_term( $tag['tag_name'], 'post_tag', $tagarr );
				if ( ! is_wp_error( $id ) ) {
					if ( isset( $tag['term_id'] ) ) {
						$this->processed_terms[ intval( $tag['term_id'] ) ] = $id['term_id'];
					}
				}
			}
			unset( $this->tags );
		}
		function process_terms() {
			$this->terms = apply_filters( 'wp_import_terms', $this->terms );
			if ( empty( $this->terms ) ) {
				return;
			}

			foreach ( $this->terms as $term ) {
				/* if the term already exists in the correct taxonomy leave it alone*/
				$term_id = term_exists( $term['slug'], $term['term_taxonomy'] );
				if ( $term_id ) {
					if ( is_array( $term_id ) ) { $term_id = $term_id['term_id'];
					}
					if ( isset( $term['term_id'] ) ) {
						$this->processed_terms[ intval( $term['term_id'] ) ] = (int) $term_id;
					}
					continue;
				}
				if ( empty( $term['term_parent'] ) ) {
					$parent = 0;
				} else {
					$parent = term_exists( $term['term_parent'], $term['term_taxonomy'] );
					if ( is_array( $parent ) ) { $parent = $parent['term_id'];
					}
				}
				$description = isset( $term['term_description'] ) ? $term['term_description'] : '';
				$termarr = array(
					'slug' => $term['slug'],
					'description' => $description,
					'parent' => intval( $parent ),
				);
				$id = wp_insert_term( $term['term_name'], $term['term_taxonomy'], $termarr );
				if ( ! is_wp_error( $id ) ) {
					if ( isset( $term['term_id'] ) ) {
						$this->processed_terms[ intval( $term['term_id'] ) ] = $id['term_id'];
					}
				}
			}
			unset( $this->terms );
		}
		function process_posts() {
			$this->posts = apply_filters( 'wp_import_posts', $this->posts );
			foreach ( $this->posts as $post ) {
				$post = apply_filters( 'wp_import_post_data_raw', $post );
				if ( isset( $this->processed_posts[ $post['post_id'] ] ) && ! empty( $post['post_id'] ) ) {
					continue;
				}
				if ( $post['status'] == 'auto-draft' ) {
					continue;
				}
				if ( 'nav_menu_item' == $post['post_type'] ) {
					$this->process_menu_item( $post );
					continue;
				}
				$post_type_object = get_post_type_object( $post['post_type'] );
				$post_exists = post_exists( $post['post_title'], '', $post['post_date'] );
				if ( $post_exists && get_post_type( $post_exists ) == $post['post_type'] ) {
					$comment_post_ID = $post_id = $post_exists;
				} else {
					$post_parent = (int) $post['post_parent'];
					if ( $post_parent ) {
						/* if we already know the parent, map it to the new local ID*/
						if ( isset( $this->processed_posts[ $post_parent ] ) ) {
							$post_parent = $this->processed_posts[ $post_parent ];
							/* otherwise record the parent for later*/
						} else {
							$this->post_orphans[ intval( $post['post_id'] ) ] = $post_parent;
							$post_parent = 0;
						}
					}
					/* map the post author*/
					$author = sanitize_user( $post['post_author'], true );
					if ( isset( $this->author_mapping[ $author ] ) ) {
						$author = $this->author_mapping[ $author ];
					} else { 					$author = (int) get_current_user_id();
					}

					$postdata = array(
					'import_id' => $post['post_id'],
					'post_author' => $author,
					'post_date' => $post['post_date'],
					'post_date_gmt' => $post['post_date_gmt'],
					'post_content' => $post['post_content'],
					'post_excerpt' => $post['post_excerpt'],
					'post_title' => $post['post_title'],
					'post_status' => $post['status'],
					'post_name' => $post['post_name'],
					'comment_status' => $post['comment_status'],
					'ping_status' => $post['ping_status'],
					'guid' => $post['guid'],
					'post_parent' => $post_parent,
					'menu_order' => $post['menu_order'],
					'post_type' => $post['post_type'],
					'post_password' => $post['post_password'],
					);

					$original_post_ID = $post['post_id'];
					$postdata = apply_filters( 'wp_import_post_data_processed', $postdata, $post );

					if ( 'attachment' == $postdata['post_type'] ) {
						$remote_url = ! empty( $post['attachment_url'] ) ? $post['attachment_url'] : $post['guid'];

						/* try to use _wp_attached file for upload folder placement to ensure the same location as the export site*/
						/* e.g. location is 2003/05/image.jpg but the attachment post_date is 2010/09, see media_handle_upload()*/
						$postdata['upload_date'] = $post['post_date'];
						if ( isset( $post['postmeta'] ) ) {
							foreach ( $post['postmeta'] as $meta ) {
								if ( $meta['key'] == '_wp_attached_file' ) {
									if ( preg_match( '%^[0-9]{4}/[0-9]{2}%', $meta['value'], $matches ) ) {
										$postdata['upload_date'] = $matches[0];
									}
									break;
								}
							}
						}
						$comment_post_ID = $post_id = $this->process_attachment( $postdata, $remote_url );
					} else {
						$comment_post_ID = $post_id = wp_insert_post( $postdata, true );
						do_action( 'wp_import_insert_post', $post_id, $original_post_ID, $postdata, $post );
					}
					if ( $post['is_sticky'] == 1 ) {
						stick_post( $post_id );
					}
				}// End if().
				/* map pre-import ID to local ID*/
				$this->processed_posts[ intval( $post['post_id'] ) ] = (int) $post_id;
				if ( ! isset( $post['terms'] ) ) {
					$post['terms'] = array();
				}
				$post['terms'] = apply_filters( 'wp_import_post_terms', $post['terms'], $post_id, $post );
				/* add categories, tags and other terms*/
				if ( ! empty( $post['terms'] ) ) {
					$terms_to_set = array();
					foreach ( $post['terms'] as $term ) {
						/* back compat with WXR 1.0 map 'tag' to 'post_tag'*/
						$taxonomy = ( 'tag' == $term['domain'] ) ? 'post_tag' : $term['domain'];
						$term_exists = term_exists( $term['slug'], $taxonomy );
						$term_id = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
						if ( ! $term_id ) {
							$t = wp_insert_term( $term['name'], $taxonomy, array(
								'slug' => $term['slug'],
							) );
							if ( ! is_wp_error( $t ) ) {
								$term_id = $t['term_id'];
								do_action( 'wp_import_insert_term', $t, $term, $post_id, $post );
							}
						}
						$terms_to_set[ $taxonomy ][] = intval( $term_id );
					}
					foreach ( $terms_to_set as $tax => $ids ) {
						$tt_ids = wp_set_post_terms( $post_id, $ids, $tax );
						do_action( 'wp_import_set_post_terms', $tt_ids, $ids, $tax, $post_id, $post );
					}
					unset( $post['terms'], $terms_to_set );
				}

				if ( ! isset( $post['comments'] ) ) {
					$post['comments'] = array();
				}

				$post['comments'] = apply_filters( 'wp_import_post_comments', $post['comments'], $post_id, $post );

				/* add/update comments*/
				if ( ! empty( $post['comments'] ) ) {
					$num_comments = 0;
					$inserted_comments = array();
					foreach ( $post['comments'] as $comment ) {
						$comment_id	= $comment['comment_id'];
						$newcomments[ $comment_id ]['comment_post_ID']      = $comment_post_ID;
						$newcomments[ $comment_id ]['comment_author']       = $comment['comment_author'];
						$newcomments[ $comment_id ]['comment_author_email'] = $comment['comment_author_email'];
						$newcomments[ $comment_id ]['comment_author_IP']    = $comment['comment_author_IP'];
						$newcomments[ $comment_id ]['comment_author_url']   = $comment['comment_author_url'];
						$newcomments[ $comment_id ]['comment_date']         = $comment['comment_date'];
						$newcomments[ $comment_id ]['comment_date_gmt']     = $comment['comment_date_gmt'];
						$newcomments[ $comment_id ]['comment_content']      = $comment['comment_content'];
						$newcomments[ $comment_id ]['comment_approved']     = $comment['comment_approved'];
						$newcomments[ $comment_id ]['comment_type']         = $comment['comment_type'];
						$newcomments[ $comment_id ]['comment_parent'] 	  = $comment['comment_parent'];
						$newcomments[ $comment_id ]['commentmeta']          = isset( $comment['commentmeta'] ) ? $comment['commentmeta'] : array();
						if ( isset( $this->processed_authors[ $comment['comment_user_id'] ] ) ) {
							$newcomments[ $comment_id ]['user_id'] = $this->processed_authors[ $comment['comment_user_id'] ];
						}
					}
					ksort( $newcomments );

					foreach ( $newcomments as $key => $comment ) {
						/* if this is a new post we can skip the comment_exists() check*/
						if ( ! $post_exists || ! comment_exists( $comment['comment_author'], $comment['comment_date'] ) ) {
							if ( isset( $inserted_comments[ $comment['comment_parent'] ] ) ) {
								$comment['comment_parent'] = $inserted_comments[ $comment['comment_parent'] ];
							}
							$comment = wp_filter_comment( $comment );
							$inserted_comments[ $key ] = wp_insert_comment( $comment );
							do_action( 'wp_import_insert_comment', $inserted_comments[ $key ], $comment, $comment_post_ID, $post );

							foreach ( $comment['commentmeta'] as $meta ) {
								$value = maybe_unserialize( $meta['value'] );
								add_comment_meta( $inserted_comments[ $key ], $meta['key'], $value );
							}

							$num_comments++;
						}
					}
					unset( $newcomments, $inserted_comments, $post['comments'] );
				}// End if().

				if ( ! isset( $post['postmeta'] ) ) {
					$post['postmeta'] = array();
				}

				$post['postmeta'] = apply_filters( 'wp_import_post_meta', $post['postmeta'], $post_id, $post );

				/* add/update post meta*/
				if ( ! empty( $post['postmeta'] ) ) {
					foreach ( $post['postmeta'] as $meta ) {
						$key = apply_filters( 'import_post_meta_key', $meta['key'], $post_id, $post );
						$value = false;
						if ( '_edit_last' == $key ) {
							if ( isset( $this->processed_authors[ intval( $meta['value'] ) ] ) ) {
								$value = $this->processed_authors[ intval( $meta['value'] ) ];
							} else { 							$key = false;
							}
						}
						if ( $key ) {
							/* export gets meta straight from the DB so could have a serialized string*/
							if ( ! $value ) {
								$value = maybe_unserialize( $meta['value'] );
							}

							add_post_meta( $post_id, $key, $value );
							do_action( 'import_post_meta', $post_id, $key, $value );
							/* if the post has a featured image, take note of this in case of remap*/
							if ( '_thumbnail_id' == $key ) {
								$this->featured_images[ $post_id ] = (int) $value;
							}
						}
					}
				}
			}// End foreach().
			unset( $this->posts );
		}
		function process_menu_item( $item ) {
			/* skip draft, orphaned menu items*/
			if ( 'draft' == $item['status'] ) {
				return;
			}
			$menu_slug = false;
			if ( isset( $item['terms'] ) ) {
				/* loop through terms, assume first nav_menu term is correct menu*/
				foreach ( $item['terms'] as $term ) {
					if ( 'nav_menu' == $term['domain'] ) {
						$menu_slug = $term['slug'];
						break;
					}
				}
			}
			/* no nav_menu term associated with this menu item*/
			if ( ! $menu_slug ) {
				echo __( 'Menu item skipped due to missing menu slug', 'templatic-admin' );
				echo '<br />';
				return;
			}
			$menu_id = term_exists( $menu_slug, 'nav_menu' );
			$menu_id = is_array( $menu_id ) ? $menu_id['term_id'] : $menu_id;
			foreach ( $item['postmeta'] as $meta ) {
				$$meta['key'] = $meta['value'];
			}

			if ( 'taxonomy' == $_menu_item_type && isset( $this->processed_terms[ intval( $_menu_item_object_id ) ] ) ) {
				$_menu_item_object_id = $this->processed_terms[ intval( $_menu_item_object_id ) ];
			} elseif ( 'post_type' == $_menu_item_type && isset( $this->processed_posts[ intval( $_menu_item_object_id ) ] ) ) {
				$_menu_item_object_id = $this->processed_posts[ intval( $_menu_item_object_id ) ];
			} elseif ( 'custom' != $_menu_item_type ) {
				/* associated object is missing or not imported yet, we'll retry later*/
				$this->missing_menu_items[] = $item;
				return;
			}
			if ( isset( $this->processed_menu_items[ intval( $_menu_item_menu_item_parent ) ] ) ) {
				$_menu_item_menu_item_parent = $this->processed_menu_items[ intval( $_menu_item_menu_item_parent ) ];
			} elseif ( $_menu_item_menu_item_parent ) {
				$this->menu_item_orphans[ intval( $item['post_id'] ) ] = (int) $_menu_item_menu_item_parent;
				$_menu_item_menu_item_parent = 0;
			}
				/* wp_update_nav_menu_item expects CSS classes as a space separated string*/
				$_menu_item_classes = maybe_unserialize( $_menu_item_classes );
			if ( is_array( $_menu_item_classes ) ) {
				$_menu_item_classes = implode( ' ', $_menu_item_classes );
			}

				$args = array(
				'menu-item-object-id' => $_menu_item_object_id,
				'menu-item-object' => $_menu_item_object,
				'menu-item-parent-id' => $_menu_item_menu_item_parent,
				'menu-item-position' => intval( $item['menu_order'] ),
				'menu-item-type' => $_menu_item_type,
				'menu-item-title' => $item['post_title'],
				'menu-item-url' => $_menu_item_url,
				'menu-item-description' => $item['post_content'],
				'menu-item-attr-title' => $item['post_excerpt'],
				'menu-item-target' => $_menu_item_target,
				'menu-item-classes' => $_menu_item_classes,
				'menu-item-xfn' => $_menu_item_xfn,
				'menu-item-status' => $item['status'],
				);
				$id = wp_update_nav_menu_item( $menu_id, 0, $args );
			if ( $id && ! is_wp_error( $id ) ) {
				$this->processed_menu_items[ intval( $item['post_id'] ) ] = (int) $id;
			}
		}
		function process_attachment( $post, $url ) {
			/* if the URL is absolute, but does not contain address, then upload it assuming base_site_url*/
			if ( preg_match( '|^/[\w\W]+$|', $url ) ) {
				$url = rtrim( $this->base_url, '/' ) . $url;
			}
			$upload = $this->fetch_remote_file( $url, $post );
			if ( is_wp_error( $upload ) ) {
				return $upload;
			}
			if ( $info = wp_check_filetype( $upload['file'] ) ) {
				$post['post_mime_type'] = $info['type'];
			} else { 			return new WP_Error( 'attachment_processing_error', __( 'Invalid file type', 'templatic-admin' ) );
			}
			$post['guid'] = $upload['url'];
			/* as per wp-admin/includes/upload.php*/
			$post_id = wp_insert_attachment( $post, $upload['file'] );
			wp_update_attachment_metadata( $post_id, wp_generate_attachment_metadata( $post_id, $upload['file'] ) );
			/* remap resized image URLs, works by stripping the extension and remapping the URL stub.*/
			if ( preg_match( '!^image/!', $info['type'] ) ) {
				$parts = pathinfo( $url );
				$name = basename( $parts['basename'], ".{$parts['extension']}" ); /* PATHINFO_FILENAME in PHP 5.2*/
				$parts_new = pathinfo( $upload['url'] );
				$name_new = basename( $parts_new['basename'], ".{$parts_new['extension']}" );
				$this->url_remap[ $parts['dirname'] . '/' . $name ] = $parts_new['dirname'] . '/' . $name_new;
			}
			return $post_id;
		}
		function fetch_remote_file( $url, $post ) {
			/* extract the file name and extension from the url*/
			$file_name = basename( $url );
			/* get placeholder file in the upload dir with a unique, sanitized filename*/
			$upload = wp_upload_bits( $file_name, 0, '', $post['upload_date'] );
			/* fetch the remote url and write it to the placeholder file*/
			$headers = wp_get_http( $url, $upload['file'] );
			/* request failed*/
			if ( ! $headers ) {
				@unlink( $upload['file'] );
			}
			/* make sure the fetch was successful*/
			if ( $headers['response'] != '200' ) {
				@unlink( $upload['file'] );
			}
			$filesize = filesize( $upload['file'] );
			if ( isset( $headers['content-length'] ) && $filesize != $headers['content-length'] ) {
				@unlink( $upload['file'] );
			}
			if ( 0 == $filesize ) {
				@unlink( $upload['file'] );
			}
			$max_size = (int) $this->max_attachment_size();
			if ( ! empty( $max_size ) && $filesize > $max_size ) {
				@unlink( $upload['file'] );
			}
			/* keep track of the old and new urls so we can substitute them later*/
			$this->url_remap[ $url ] = $upload['url'];
			$this->url_remap[ $post['guid'] ] = $upload['url']; /* r13735, really needed?*/
			/* keep track of the destination if the remote url is redirected somewhere else*/
			if ( isset( $headers['x-final-location'] ) && $headers['x-final-location'] != $url ) {
				$this->url_remap[ $headers['x-final-location'] ] = $upload['url'];
			}

			return $upload;
		}
		function backfill_parents() {
			global $wpdb;

			/* find parents for post orphans*/
			foreach ( $this->post_orphans as $child_id => $parent_id ) {
				$local_child_id = $local_parent_id = false;
				if ( isset( $this->processed_posts[ $child_id ] ) ) {
					$local_child_id = $this->processed_posts[ $child_id ];
				}
				if ( isset( $this->processed_posts[ $parent_id ] ) ) {
					$local_parent_id = $this->processed_posts[ $parent_id ];
				}

				if ( $local_child_id && $local_parent_id ) {
					$wpdb->update( $wpdb->posts, array(
						'post_parent' => $local_parent_id,
						), array(
						'ID' => $local_child_id,
					), '%d', '%d' );
				}
			}

			/* all other posts/terms are imported, retry menu items with missing associated object*/
			$missing_menu_items = $this->missing_menu_items;
			foreach ( $missing_menu_items as $item ) {
				$this->process_menu_item( $item );
			}

			/* find parents for menu item orphans*/
			foreach ( $this->menu_item_orphans as $child_id => $parent_id ) {
				$local_child_id = $local_parent_id = 0;
				if ( isset( $this->processed_menu_items[ $child_id ] ) ) {
					$local_child_id = $this->processed_menu_items[ $child_id ];
				}
				if ( isset( $this->processed_menu_items[ $parent_id ] ) ) {
					$local_parent_id = $this->processed_menu_items[ $parent_id ];
				}

				if ( $local_child_id && $local_parent_id ) {
					update_post_meta( $local_child_id, '_menu_item_menu_item_parent', (int) $local_parent_id );
				}
			}
		}
		function backfill_attachment_urls() {
			global $wpdb;
			/* make sure we do the longest urls first, in case one is a substring of another*/
			uksort( $this->url_remap, array( &$this, 'cmpr_strlen' ) );

			foreach ( $this->url_remap as $from_url => $to_url ) {
				/* remap urls in post_content*/
				$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->posts} SET post_content = REPLACE(post_content, %s, %s)", $from_url, $to_url ) );
				/* remap enclosure urls*/
				$result = $wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->postmeta} SET meta_value = REPLACE(meta_value, %s, %s) WHERE meta_key='enclosure'", $from_url, $to_url ) );
			}
		}
		function remap_featured_images() {
			/* cycle through posts that have a featured image*/
			foreach ( $this->featured_images as $post_id => $value ) {
				if ( isset( $this->processed_posts[ $value ] ) ) {
					$new_id = $this->processed_posts[ $value ];
					/* only update if there's a difference*/
					if ( $new_id != $value ) {
						update_post_meta( $post_id, '_thumbnail_id', $new_id );
					}
				}
			}
		}
		function parse( $file ) {
			$parser = new WXR_Parser();
			return $parser->parse( $file );
		}
		function is_valid_meta_key( $key ) {
			/* skip attachment metadata since we'll regenerate it from scratch*/
			/* skip _edit_lock as not relevant for import*/
			if ( in_array( $key, array( '_wp_attached_file', '_wp_attachment_metadata', '_edit_lock' ) ) ) {
				return false;
			}
			return $key;
		}
		function allow_create_users() {
			return apply_filters( 'import_allow_create_users', true );
		}
		function allow_fetch_attachments() {
			return apply_filters( 'import_allow_fetch_attachments', true );
		}
		function max_attachment_size() {
			return apply_filters( 'import_attachment_size_limit', 0 );
		}
		function bump_request_timeout() {
			return 60;
		}
		function cmpr_strlen( $a, $b ) {
			return strlen( $b ) - strlen( $a );
		}
	}
} // End if().


$xml_import = new Listing_WP_Import();

if ( file_exists( WP_PLUGIN_DIR . '/Tevolution-Directory/listing-dummy-data.xml' ) ) {
	$xml_import->import( WP_PLUGIN_DIR . '/Tevolution-Directory/listing-dummy-data.xml' );
	update_option( 'listing_xml',1 );

	$sidebars_widgets = get_option( 'sidebars_widgets' );  /*collect widget informations*/
	//==============================LISTING SIDEBAR WIDGET AREA SETTING START=========================//
	$directory_search_location_key2 = '';
	//Search widget settings start
	$directory_search_location[88] = array(
		'title'				=> __( 'Search Nearby Listings','templatic' ),
		'post_type'			=> array( 'listing' ),
		'miles_search'		=> 0,
		'radius_measure'	=> 'miles',
		);
	$directory_search_location['_multiwidget'] = '1';
	$directory_search_location = get_option( 'widget_directory_search_location' );
	update_option( 'widget_directory_search_location', $directory_search_location );
	$directory_search_location = get_option( 'widget_directory_search_location' );
	if ( is_array( $directory_search_location ) && ! empty( $directory_search_location ) ) {
		krsort( $directory_search_location );
		foreach ( $directory_search_location as $key1 => $val1 ) {
			$directory_search_location_key2 = $key1;
			if ( is_int( $directory_search_location_key2 ) ) {
				break;
			}
		}
	}
	//Search widget settings end

	//T → Search Near By Miles Range widget settings start
	$directory_mile_range_widget = array();
	$directory_mile_range_widget[1] = array(
		'title'				=> __( 'Filter Listings By Miles','templatic' ),
		'max_range'			=> 500,
		'post_type'			=> 'listing',
		);
	$directory_mile_range_widget['_multiwidget'] = '1';
	update_option( 'widget_directory_mile_range_widget', $directory_mile_range_widget );
	$directory_mile_range_widget = get_option( 'widget_directory_mile_range_widget' );
	krsort( $directory_mile_range_widget );
	foreach ( $directory_mile_range_widget as $key1 => $val1 ) {
		$directory_mile_range_widget_key = $key1;
		if ( is_int( $directory_mile_range_widget_key ) ) {
			break;
		}
	}
	//T → Search Near By Miles Range widget settings end

	//Browse by category widget settings start
	$templatic_browse_by_categories = array();
	$templatic_browse_by_categories[1] = array(
		'title'				=> __( 'Browse Listings By Category','templatic' ),
		'post_type'			=> 'listing',
		'categories_count'	=> 1,
		);
	$templatic_browse_by_categories['_multiwidget'] = '1';
	update_option( 'widget_templatic_browse_by_categories', $templatic_browse_by_categories );
	$templatic_browse_by_categories = get_option( 'widget_templatic_browse_by_categories' );
	krsort( $templatic_browse_by_categories );
	foreach ( $templatic_browse_by_categories as $key1 => $val1 ) {
		$templatic_browse_by_categories_key1 = $key1;
		if ( is_int( $templatic_browse_by_categories_key1 ) ) {
			break;
		}
	}
	//Browse by category widget settings end
	//Advertisement widget settings start
	$templatic_text[87] = array(
		'title'	=> '',
		'text'	=> '<a href="http://templatic.com/docs/directory-theme-guide/"><img align="middle" src="http://demo.templatic.com/directory/images/directory.jpg"></a>',
		);
	$templatic_text['_multiwidget'] = '1';
	$templatic_text = get_option( 'widget_templatic_text' );
	update_option( 'widget_templatic_text', $templatic_text );
	$templatic_text = get_option( 'widget_templatic_text' );
	$templatic_text_key4 = '';
	if ( is_array( $templatic_text ) ) {
		krsort( $templatic_text );
		foreach ( $templatic_text as $key => $val ) {
			$templatic_text_key4 = $key;
			if ( is_int( $templatic_text_key4 ) ) {
				break;
			}
		}
	}
	//advertisement widget settings end
	$sidebars_widgets['listingcategory_listing_sidebar'] = array( "directory_search_location-{$directory_search_location_key2}","directory_mile_range_widget-{$directory_mile_range_widget_key}", "templatic_browse_by_categories-{$templatic_browse_by_categories_key1}","templatic_text-{$templatic_text_key4}" );
	//==============================LISTING SIDEBAR WIDGET AREA SETTING END=========================//
	//==============================LISTING DETAIL SIDEBAR WIDGET AREA SETTING END=========================//
	//T → In the neighborhood widget settings start
	$directory_neighborhood = array();
	$directory_neighborhood[1] = array(
		'title'					=> __( 'Nearest Listings','templatic' ),
		'post_type'				=> 'listing',
		'post_number'			=> 4,
		'content_limit'			=> 34,
		'show_list'				=> 0,
		'closer_factor'			=> 0,
		'radius'				=> 5000,
		'radius_measure'		=> 'miles',
		);
	$directory_neighborhood['_multiwidget'] = '1';
	update_option( 'widget_directory_neighborhood', $directory_neighborhood );
	$directory_neighborhood = get_option( 'widget_directory_neighborhood' );
	krsort( $directory_neighborhood );
	foreach ( $directory_neighborhood as $key1 => $val1 ) {
		$directory_neighborhood_key1 = $key1;
		if ( is_int( $directory_neighborhood_key1 ) ) {
			break;
		}
	}
	//T → In the neighborhood widget settings end

	//Search widget settings start
	$directory_search_location[89] = array(
		'title'				=> __( 'Search Nearby Listings','templatic' ),
		'post_type'			=> array( 'listing' ),
		'miles_search'		=> 0,
		'radius_measure'	=> 'miles',
		);
	$directory_search_location['_multiwidget'] = '1';
	$directory_search_location = get_option( 'widget_directory_search_location' );
	update_option( 'widget_directory_search_location', $directory_search_location );
	$directory_search_location = get_option( 'widget_directory_search_location' );
	$directory_search_location_key3 = '';
	if ( is_array( $directory_search_location ) && ! empty( $directory_search_location ) ) {
		krsort( $directory_search_location );
		foreach ( $directory_search_location as $key1 => $val1 ) {
			$directory_search_location_key3 = $key1;
			if ( is_int( $directory_search_location_key3 ) ) {
				break;
			}
		}
	}
	//Search widget settings end

	//Browse by category widget settings start
	$templatic_browse_by_categories[2] = array(
		'title'				=> __( 'Browse Listings By Categories','templatic' ),
		'post_type'			=> 'listing',
		'categories_count'	=> 1,
		);
	$templatic_browse_by_categories['_multiwidget'] = '1';
	update_option( 'widget_templatic_browse_by_categories', $templatic_browse_by_categories );
	$templatic_browse_by_categories = get_option( 'widget_templatic_browse_by_categories' );
	krsort( $templatic_browse_by_categories );
	foreach ( $templatic_browse_by_categories as $key1 => $val1 ) {
		$templatic_browse_by_categories_key2 = $key1;
		if ( is_int( $templatic_browse_by_categories_key2 ) ) {
			break;
		}
	}
	//Browse by category widget settings end

	//Advertisement widget settings start
	$templatic_text[86] = array(
		'title'	=> '',
		'text'	=> '<a href="http://templatic.com"><img align="middle" src="' . get_template_directory_uri() . '/images/adv_300x250.jpg"></a>',
		);
	$templatic_text['_multiwidget'] = '1';
	$templatic_text = get_option( 'widget_templatic_text' );
	update_option( 'widget_templatic_text', $templatic_text );
	$templatic_text = get_option( 'widget_templatic_text' );
	$templatic_text_key4 = '';
	if ( is_array( $templatic_text ) && ! empty( $templatic_text ) ) {
		krsort( $templatic_text );
		foreach ( $templatic_text as $key => $val ) {
			$templatic_text_key4 = $key;
			if ( is_int( $templatic_text_key4 ) ) {
				break;
			}
		}
	}
	//advertisement widget settings end
	//Advertisement widget settings start
	$templatic_text[85] = array(
		'title'	=> '',
		'text'	=> '<a href="http://templatic.com/docs/directory-theme-guide/"><img align="middle" src="http://demo.templatic.com/directory/images/directory.jpg"></a>',
		);
	$templatic_text['_multiwidget'] = '1';
	$templatic_text = get_option( 'widget_templatic_text' );
	update_option( 'widget_templatic_text', $templatic_text );
	$templatic_text = get_option( 'widget_templatic_text' );
	$templatic_text_key5 = '';
	if ( is_array( $templatic_text ) && ! empty( $templatic_text ) ) {
		krsort( $templatic_text );
		foreach ( $templatic_text as $key => $val ) {
			$templatic_text_key5 = $key;
			if ( is_int( $templatic_text_key5 ) ) {
				break;
			}
		}
	}
	//advertisement widget settings end
	$sidebars_widgets['listing_detail_sidebar'] = array( "directory_neighborhood-{$directory_neighborhood_key1}", "templatic_text-{$templatic_text_key4}", "directory_search_location-{$directory_search_location_key3}", "templatic_browse_by_categories-{$templatic_browse_by_categories_key2}", "templatic_text-{$templatic_text_key5}" );
	//==============================LISTING DETAIL SIDEBAR WIDGET AREA SETTING END=========================//

	update_option( 'sidebars_widgets', $sidebars_widgets );  /*save widget informations */
	update_option( 'hide_listing_ajax_notification', true );
	add_action( 'admin_notices','listing_sample_data_inserted' );

	/* Add package for all sample listing */
	do_action( 'tmpl_add_sample_listing_package',CUSTOM_POST_TYPE_LISTING );
}// End if().
function listing_sample_data_inserted() {
	?>
	<div class="classified-welcome-panel">
		<div class="tmpl-auto-install-yb">
			<?php _e( 'Sample Data has been inserted.','templatic' ); ?>
		</div>
	</div>
	<?php
}
?>
